@@ -1,13 +1,13 @@
 /* apt-dater - terminal-based remote package update manager
  *
- * $Id: apt-dater.h 206 2008-11-17 13:34:41Z liske $
+ * $Id: apt-dater.h 358 2009-05-25 11:39:38Z liske $
  *
  * Authors:
  *   Andre Ellguth <ellguth@ibh.de>
  *   Thomas Liske <liske@ibh.de>
  *
  * Copyright Holder:
- *   2008 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
+ *   2008-2009 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
  *
  * License:
  *   This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,9 @@
 #ifndef _APT_DATER_H
 #define _APT_DATER_H
 
+#define _GNU_SOURCE
+
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -37,6 +40,20 @@
 #include <sys/stat.h>
 
 #include <glib-2.0/glib.h>
+#include <glib-2.0/glib/gstdio.h>
+
+#include "../config.h"
+
+
+#ifdef HAVE_GETTEXT
+
+#include <libintl.h>
+#define _(x) gettext(x)
+#else
+#define _(x) x
+#endif
+#define N_(x) x
+
 
 #define STATS_MAX_LINE_LEN 1000
 #define INPUT_MAX 4096
@@ -48,7 +65,32 @@
 # include "config.h"
 #endif
 
+#ifndef NDEBUG
+typedef enum {
+    T_CFGFILE=1,
+    T_DRAWNODE=2,
+    T_SESSNODE=3,
+    T_HOSTNODE=4,
+    T_PKGNODE=5,
+    T_MAPPING=6,
+    T_VERSION=7,
+    T_DISTRI=8,
+} etype;
+
+#define ASSERT_TYPE(p,t) \
+    if ((p)->_type != (t)) \
+	g_error("Unexpected type %d for " #p " in " __FILE__ ":%d" \
+		", should be " #t "(%d)!\n", (p)->_type, __LINE__, (t));
+#else
+
+#define ASSERT_TYPE(p,t)
+
+#endif
+
 typedef struct _cfgfile {
+#ifndef NDEBUG
+ etype _type;
+#endif
  gchar *hostsfile;
  gchar *screenrcfile;
  gchar *screentitle;
@@ -58,20 +100,35 @@ typedef struct _cfgfile {
  gchar *ssh_optflags;
  gchar *ssh_defuser;
  gint  ssh_defport;
+ gboolean ssh_agent;
+ gchar **ssh_add;
+ gsize ssh_numadd;
  gchar *cmd_refresh;
  gchar *cmd_upgrade;
  gchar *cmd_install;
  gboolean use_screen;
  gboolean dump_screen;
  gboolean query_maintainer;
+ gboolean beep;
+ gboolean flash;
+#ifdef FEAT_HISTORY
+ gboolean record_history;
+ gchar *history_errpattern;
+#endif
  gchar **colors;
 #ifdef FEAT_TCLFILTER
  gchar *filterexp;
  gchar *filterfile;
 #endif
+#ifdef FEAT_AUTOREF
+ gboolean auto_refresh;
+#endif
 } CfgFile;
 
 typedef struct _update {
+#ifndef NDEBUG
+ etype _type;
+#endif
  gchar *package;
  gchar *version;
  gint flag;
@@ -79,6 +136,9 @@ typedef struct _update {
 } PkgNode;
 
 typedef struct _session {
+#ifndef NDEBUG
+ etype _type;
+#endif
   gint pid;
   struct stat st;
 } SessNode;
@@ -86,53 +146,75 @@ typedef struct _session {
 typedef enum {
  C_UPDATES_PENDING = 0,
  C_UP_TO_DATE = 1,
- C_NO_STATS = 2,
+ C_BROKEN_PKGS = 2,
  C_REFRESH_REQUIRED = 3,
  C_REFRESH = 4,
  C_SESSIONS = 5,
 #ifdef FEAT_TCLFILTER
  C_FILTERED = 6,
- C_UNKNOW = 7,
+ C_UNKNOWN = 7,
 #else
- C_UNKNOW = 6,
+ C_UNKNOWN = 6,
 #endif
 } Category;
 
-#define HOST_STATUS_PKGUPDATE        1
-#define HOST_STATUS_PKGKEPTBACK      2
-#define HOST_STATUS_PKGEXTRA         4
-#define HOST_STATUS_KERNELNOTMATCH   8
-#define HOST_STATUS_KERNELSELFBUILD 16
-#define HOST_STATUS_VIRTUALIZED     32
-#define HOST_STATUS_LOCKED          64
+#define HOST_STATUS_PKGUPDATE         1
+#define HOST_STATUS_PKGKEPTBACK       2
+#define HOST_STATUS_PKGEXTRA          4
+#define HOST_STATUS_PKGBROKEN         8
+#define HOST_STATUS_KERNELNOTMATCH   16
+#define HOST_STATUS_KERNELSELFBUILD  32
+#define HOST_STATUS_VIRTUALIZED      64
+#define HOST_STATUS_LOCKED          128
+
+#define HOST_FORBID_REFRESH           1
+#define HOST_FORBID_UPGRADE           2
+#define HOST_FORBID_INSTALL           4
+
+#define HOST_FORBID_MASK     (HOST_FORBID_REFRESH | HOST_FORBID_UPGRADE | HOST_FORBID_INSTALL)
 
 typedef struct _hostnode {
+#ifndef NDEBUG
+ etype _type;
+#endif
  gchar     *hostname;
  gchar     *group;
  gchar     *ssh_user;
  gint      ssh_port;
+#ifdef FEAT_AUTOREF
+ time_t    last_upd;
+#endif
  guint     status;
  gboolean  keptback;
 #ifdef FEAT_TCLFILTER
  gboolean  filtered;
+#endif
+#ifdef FEAT_HISTORY
+ gboolean  parse_result;
+ gint      hist_ts;
 #endif
  Category  category;
  GList     *packages;
  gint      nupdates;
  gint      nholds;
  gint      nextras;
+ gint      nbrokens;
  GList     *screens;
  gint      fdlock;
+ FILE      *fpstat;
  gchar     *lsb_distributor;
  gchar     *lsb_release;
  gchar     *lsb_codename;
+ gchar     *uname_kernel;
+ gchar     *uname_machine;
  gchar     *kernelrel;
  gchar     *virt;
  gchar     *identity_file;
+ gint      forbid;
+ gboolean  tagged;
 } HostNode;
 
-struct mapping_t
-{
+struct mapping_t {
  gchar *name;
  gint value;
 };
