@@ -1,13 +1,13 @@
 /* apt-dater - terminal-based remote package update manager
  *
- * $Id: exec.c 204 2008-11-10 15:39:28Z ellguth $
+ * $Id: exec.c 358 2009-05-25 11:39:38Z liske $
  *
  * Authors:
  *   Andre Ellguth <ellguth@ibh.de>
  *   Thomas Liske <liske@ibh.de>
  *
  * Copyright Holder:
- *   2008 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
+ *   2008-2009 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
  *
  * License:
  *   This program is free software; you can redistribute it and/or modify
@@ -30,6 +30,7 @@
 #include "exec.h"
 #include "stats.h"
 #include "parsecmd.h"
+#include "history.h"
 #include <glib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -40,7 +41,8 @@
 # include "config.h"
 #endif
 
-gboolean ssh_cmd_refresh(HostNode *n)
+gboolean
+ssh_cmd_refresh(HostNode *n)
 {
  gboolean r;
  GError *error = NULL;
@@ -52,18 +54,17 @@ gboolean ssh_cmd_refresh(HostNode *n)
  gint  standard_output;
  GIOChannel *iocstdout;
 
- if(!n) return (FALSE);
+ g_assert(n);
 
- cmd = g_strdup_printf("%s+-n+-o+BatchMode=yes+-o+ConnectTimeout=5+-q+-l+%s+-p+%d%s+%s+unset LANG && %s",
+ cmd = g_strdup_printf("%s+-n+-o+BatchMode=yes+-o+ConnectTimeout=5+-q+-l+%s+-p+%d%s+%s+%s",
 		       cfg->ssh_cmd, n->ssh_user, n->ssh_port, 
 		       n->identity_file && strlen(n->identity_file) > 0 ? (identity_file = g_strconcat("+-i+", n->identity_file , NULL)) : "",
 		       n->hostname, cfg->cmd_refresh);
  g_free(identity_file);
- if(!cmd) return(FALSE);
 
  argv = g_strsplit(cmd, "+", 0);
 
- removeStatsFile(n->hostname);
+ prepareStatsFile(n);
 
  r = g_spawn_async_with_pipes(g_getenv ("HOME"), /* working_directory */
 			      argv,
@@ -97,12 +98,13 @@ gboolean ssh_cmd_refresh(HostNode *n)
  g_free(output);
  g_free(cmd);
  g_strfreev(argv);
- 
- return (r);
+
+ return r;
 }
 
 
-gboolean ssh_cmd_upgrade(HostNode *n, const gboolean detached)
+gboolean
+ssh_cmd_upgrade(HostNode *n, const gboolean detached)
 {
  gboolean r;
  GError *error = NULL;
@@ -111,19 +113,32 @@ gboolean ssh_cmd_upgrade(HostNode *n, const gboolean detached)
  gchar *identity_file = NULL;
  gchar **argv = NULL;
 
- gchar *screen = screen_new(n, detached);
+ g_assert(n);
 
- cmd = g_strdup_printf ("%s%s+-l+%s+-p+%d%s%s+%s+unset LANG ; export MAINTAINER='%s' ; %s", 
+ if(n->forbid & HOST_FORBID_UPGRADE)
+    return FALSE;
+
+ HistoryEntry he;
+ he.ts = time(NULL);
+ he.maintainer = maintainer;
+ he.action = "upgrade";
+ he.data = NULL;
+
+ gchar *screen = screen_new(n, detached, &he);
+
+ cmd = g_strdup_printf ("%s%s+-l+%s+-p+%d%s%s+%s+%s", 
 			screen,
 			cfg->ssh_cmd, n->ssh_user, n->ssh_port, 
 			cfg->ssh_optflags && strlen(cfg->ssh_optflags) > 0 ? (optflags = g_strconcat("+", cfg->ssh_optflags , NULL)) : "",
 			n->identity_file && strlen(n->identity_file) > 0 ? (identity_file = g_strconcat("+-i+", n->identity_file , NULL)) : "",
-			n->hostname, maintainer, cfg->cmd_upgrade);
+			n->hostname, cfg->cmd_upgrade);
  g_free(optflags);
  g_free(identity_file);
  g_free(screen);
 
- if(!cmd) return(FALSE);
+#ifdef FEAT_HISTORY
+ n->parse_result = cfg->history_errpattern && strlen(cfg->history_errpattern);;
+#endif
 
  argv = g_strsplit(cmd, "+", 0);
 
@@ -139,11 +154,18 @@ gboolean ssh_cmd_upgrade(HostNode *n, const gboolean detached)
  g_free(cmd);
  g_strfreev(argv);
 
- return (r);
+#ifdef FEAT_HISTORY
+ if(!detached && n->parse_result && !screen_get_sessions(n)) {
+    n->parse_result = FALSE;
+    return history_ts_failed(n);
+ }
+#endif
+
+ return FALSE;
 }
 
-
-gboolean ssh_cmd_install(HostNode *n, const gchar *package, const gboolean detached)
+gboolean
+ssh_cmd_install(HostNode *n, gchar *package, const gboolean detached)
 {
  gboolean r;
  GError *error = NULL;
@@ -153,25 +175,37 @@ gboolean ssh_cmd_install(HostNode *n, const gchar *package, const gboolean detac
  gchar *identity_file = NULL;
  gchar **argv = NULL;
 
- gchar *screen = screen_new(n, detached);
+ g_assert(n);
 
- cmd = g_strdup_printf ("%s%s+-l+%s+-p+%d%s%s+%s+unset LANG ; export MAINTAINER='%s' ; %s", 
+ if(n->forbid & HOST_FORBID_INSTALL)
+    return FALSE;
+
+ HistoryEntry he;
+ he.ts = time(NULL);
+ he.maintainer = maintainer;
+ he.action = "install";
+ he.data = package;
+
+ gchar *screen = screen_new(n, detached, &he);
+
+ cmd = g_strdup_printf ("%s%s+-l+%s+-p+%d%s%s+%s+%s", 
 			screen,
 			cfg->ssh_cmd, n->ssh_user, n->ssh_port, 
 			cfg->ssh_optflags && strlen(cfg->ssh_optflags) > 0 ? (optflags = g_strconcat("+", cfg->ssh_optflags , NULL)) : "",
 			n->identity_file && strlen(n->identity_file) > 0 ? (identity_file = g_strconcat("+-i+", n->identity_file , NULL)) : "",
-			n->hostname, maintainer, cfg->cmd_install);
+			n->hostname, cfg->cmd_install);
  g_free(optflags);
  g_free(identity_file);
  g_free(screen);
 
  buf = g_strdup_printf (cmd, package);
- if(!buf) return(FALSE);
- 
+
  g_free(cmd);
  cmd = buf;
- 
- if(!cmd) return(FALSE);
+
+#ifdef FEAT_HISTORY
+ n->parse_result = cfg->history_errpattern && strlen(cfg->history_errpattern);
+#endif
 
  argv = g_strsplit(cmd, "+", 0);
 
@@ -186,11 +220,18 @@ gboolean ssh_cmd_install(HostNode *n, const gchar *package, const gboolean detac
 
  g_free(cmd);
  g_strfreev(argv);
- 
- return (r);
+
+#ifdef FEAT_HISTORY
+ if(!detached && n->parse_result && !screen_get_sessions(n)) {
+    n->parse_result = FALSE;
+    return history_ts_failed(n);
+ }
+#endif
+
+ return FALSE;
 }
 
-gboolean ssh_connect(HostNode *n, const gboolean detached)
+void ssh_connect(HostNode *n, const gboolean detached)
 {
  gboolean r;
  GError *error = NULL;
@@ -199,20 +240,24 @@ gboolean ssh_connect(HostNode *n, const gboolean detached)
  gchar *identity_file = NULL;
  gchar **argv = NULL;
 
- gchar *screen = screen_new(n, detached);
+ HistoryEntry he;
+ he.ts = time(NULL);
+ he.maintainer = maintainer;
+ he.action = "connect";
+ he.data = NULL;
 
- cmd = g_strdup_printf ("%s%s+-l+%s+-t+-p+%d%s%s+%s+export MAINTAINER='%s' ; $SHELL",
+ gchar *screen = screen_new(n, detached, &he);
+
+ cmd = g_strdup_printf ("%s%s+-l+%s+-t+-p+%d%s%s+%s",
 			screen,
 			cfg->ssh_cmd, n->ssh_user, n->ssh_port, 
 			cfg->ssh_optflags && strlen(cfg->ssh_optflags) > 0 ? (optflags = g_strconcat("+", cfg->ssh_optflags , NULL)) : "",
 			n->identity_file && strlen(n->identity_file) > 0 ? (identity_file = g_strconcat("+-i+", n->identity_file , NULL)) : "",
-			n->hostname, maintainer);
+			n->hostname);
  g_free(optflags);
  g_free(identity_file);
  g_free(screen);
 
- if(!cmd) return(FALSE);
-
  argv = g_strsplit(cmd, "+", 0);
 
  r = g_spawn_sync(g_getenv ("HOME"), argv, NULL, 
@@ -226,26 +271,31 @@ gboolean ssh_connect(HostNode *n, const gboolean detached)
 
  g_free(cmd);
  g_strfreev(argv);
- 
- return (r);
 }
 
-gboolean sftp_connect(HostNode *n)
+void sftp_connect(HostNode *n)
 {
  gboolean r;
  GError *error = NULL;
  gint argc = 0;
  gchar **argv = NULL;
 
+ g_assert(n);
 
- gchar *cmd = screen_new(n, FALSE);
+ HistoryEntry he;
+ he.ts = time(NULL);
+ he.maintainer = maintainer;
+ he.action = "sftp";
+ he.data = NULL;
+
+ gchar *cmd = screen_new(n, FALSE, &he);
 
  cmd = g_realloc(cmd, strlen(cmd) + strlen(cfg->sftp_cmd) + 1);
  strcat(cmd, cfg->sftp_cmd);
 
  if (parse_cmdline(cmd, &argc, &argv, n) < 0) {
    g_free(cmd);
-   return FALSE;
+   return;
  }
  g_free(cmd);
 
@@ -264,6 +314,4 @@ gboolean sftp_connect(HostNode *n)
  }
 
  g_strfreev(argv);
- 
- return (r);
 }
