@@ -1,13 +1,13 @@
 /* apt-dater - terminal-based remote package update manager
  *
- * $Id: screen.c 200 2008-08-27 18:30:15Z liske $
+ * $Id: screen.c 358 2009-05-25 11:39:38Z liske $
  *
  * Authors:
  *   Andre Ellguth <ellguth@ibh.de>
  *   Thomas Liske <liske@ibh.de>
  *
  * Copyright Holder:
- *   2008 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
+ *   2008-2009 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
  *
  * License:
  *   This program is free software; you can redistribute it and/or modify
@@ -38,8 +38,8 @@
 
 #include "screen.h"
 #include "parsecmd.h"
+#include "history.h"
 
-static gchar *dump_fn = NULL;
 static struct passwd *pw = NULL;
 
 const static gchar *
@@ -52,7 +52,7 @@ screen_get_sdir() {
     return NULL;
 
   g_snprintf(sdir, sizeof(sdir), SCREEN_SDFORMT, SCREEN_SOCKDIR, pw->pw_name);
-  
+
   return sdir;
 }
 
@@ -77,7 +77,7 @@ screen_get_sessions(HostNode *n) {
   const gchar *f;
   while ((f = g_dir_read_name(d))) {
     gchar *fn = g_strdup_printf("%s/%s", sdir, f);
-    
+
     if (g_file_test(fn, G_FILE_TEST_EXISTS)) {
       gint pid = atoi(f);
       char *name = strchr(f, '.');
@@ -87,6 +87,9 @@ screen_get_sessions(HostNode *n) {
 	  (strcmp(name+1, search) == 0)) {
 
 	SessNode *s = g_new0(SessNode, 1);
+#ifndef NDEBUG
+	s->_type = T_SESSNODE;
+#endif
 	s->pid = pid;
 	stat(fn, &s->st);
 
@@ -104,17 +107,54 @@ screen_get_sessions(HostNode *n) {
 }
 
 gchar *
-screen_new(HostNode *n, const gboolean detached) {
-  if (!cfg->use_screen)
-    return g_strdup("");
-  
+screen_new(HostNode *n, const gboolean detached, const HistoryEntry *he) {
+  if (!cfg->use_screen) {
+#ifdef FEAT_HISTORY
+    if(cfg->record_history && he) {
+     gchar *hp = history_rec_path(n);
+     gchar *fn_meta = g_strdup_printf("%s/meta", hp);
+     history_write_meta(fn_meta, he);
+     g_free(fn_meta);
+
+     gchar *cmd = g_strdup_printf(PKGLIBDIR"/script+%s+%s+", hp, cfg->history_errpattern);
+     g_free(hp);
+
+     return cmd;
+    }
+    else
+#endif
+     return g_strdup("");
+  }
+
   gchar *title = parse_string(cfg->screentitle, n);
 
-  gchar *cmd = g_strdup_printf(SCREEN_BINARY"+-%sS+"SCREEN_SOCKPRE"%s_%s_%d"	\
+  gchar *cmd;
+
+#ifdef FEAT_HISTORY
+  if(cfg->record_history && he) {
+    gchar *hp = history_rec_path(n);
+    gchar *fn_meta = g_strdup_printf("%s/meta", hp);
+    history_write_meta(fn_meta, he);
+    g_free(fn_meta);
+
+    cmd = g_strdup_printf(SCREEN_BINARY"+-%sS+"SCREEN_SOCKPRE"%s_%s_%d"	\
+			       "+-t+%s+-c+%s+"PKGLIBDIR"/script+%s+%s+",
+			       detached ? "dm" : "",
+			       n->ssh_user, n->hostname, n->ssh_port,
+			       title,
+			       cfg->screenrcfile,
+			       hp, cfg->history_errpattern);
+
+    g_free(hp);
+  } else
+#endif
+    cmd = g_strdup_printf(SCREEN_BINARY"+-%sS+"SCREEN_SOCKPRE"%s_%s_%d"	\
 			       "+-t+%s+-c+%s+",
 			       detached ? "dm" : "",
 			       n->ssh_user, n->hostname, n->ssh_port,
-			       title, cfg->screenrcfile);
+			       title,
+			       cfg->screenrcfile);
+
   g_free(title);
 
   return cmd;
@@ -126,13 +166,13 @@ screen_attach_cmd(const SessNode *s, const gboolean shared) {
 }
 
 gboolean
-screen_attach(const SessNode *s, const gboolean shared) {
+screen_attach(HostNode *n, const SessNode *s, const gboolean shared) {
  gboolean r;
  GError *error = NULL;
  gchar *cmd = screen_attach_cmd(s, shared);
  gchar **argv = NULL;
 
- if(!cmd) return FALSE;
+ g_assert(n);
 
  argv = g_strsplit(cmd, "+", 0);
 
@@ -147,8 +187,15 @@ screen_attach(const SessNode *s, const gboolean shared) {
 
  g_free(cmd);
  g_strfreev(argv);
- 
- return r;
+
+#ifdef FEAT_HISTORY
+ if(n->parse_result && !screen_get_sessions(n)) {
+    n->parse_result = FALSE;
+    return history_ts_failed(n);
+ }
+#endif
+
+ return FALSE;
 }
 
 static gchar *
@@ -162,18 +209,15 @@ screen_get_dump(const SessNode *s) {
  GError *error = NULL;
  gchar **argv = NULL;
 
- if(!dump_fn)
-   dump_fn = g_strdup_printf("/tmp/%d.dump", getpid());
-
- g_unlink(dump_fn);
-
- gint fd = g_open(dump_fn, O_CREAT | O_EXCL | O_TRUNC |O_RDWR, S_IRUSR | S_IWUSR);
+ gchar *dump_fn = g_strdup_printf("%s/dump-XXXXXX", g_get_tmp_dir());
+ gint fd = g_mkstemp(dump_fn);
 
  if(fd == -1)
    return NULL;
 
  gchar *cmd = screen_dump_cmd(s, dump_fn);
  if(!cmd) {
+  g_unlink(dump_fn);
   close(fd);
   return NULL;
  }
@@ -191,11 +235,13 @@ screen_get_dump(const SessNode *s) {
 
  g_free(cmd);
  g_strfreev(argv);
- 
+
  gchar *c = NULL;
  g_file_get_contents(dump_fn, &c, NULL, NULL);
 
  close(fd);
 
+ g_unlink(dump_fn);
+
  return c;
 }
