@@ -1,13 +1,13 @@
 /* apt-dater - terminal-based remote package update manager
  *
- * $Id: stats.c 207 2008-11-17 15:53:39Z ellguth $
+ * $Id: stats.c 364 2009-05-26 07:23:14Z ellguth $
  *
  * Authors:
  *   Andre Ellguth <ellguth@ibh.de>
  *   Thomas Liske <liske@ibh.de>
  *
  * Copyright Holder:
- *   2008 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
+ *   2008-2009 (C) IBH IT-Service GmbH [http://www.ibh.de/apt-dater/]
  *
  * License:
  *   This program is free software; you can redistribute it and/or modify
@@ -32,81 +32,52 @@
 #include "screen.h"
 #include "stats.h"
 #include "lock.h"
+#include "autoref.h"
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
-gchar *getStatsFile(const gchar *hostname)
+gchar *getStatsFile(const HostNode *n)
 {
- GDir *dir;
- GError *error = NULL;
- const gchar *entry;
- GFileTest test;
- gchar *compare;
- gchar *statsfile = NULL;
- 
- if(!(dir = g_dir_open(cfg->statsdir, 0, &error))) {
-  g_error ("%s: %s", cfg->statsdir, error->message);
-  return (NULL);
- }
- g_clear_error(&error);
+ gchar *statsfile = getStatsFileName(n);
 
- while(((entry = g_dir_read_name(dir)))) {
-  compare = g_strdup_printf("%s.stat", hostname);
-  if(!g_strcasecmp(entry, compare)) {
-   statsfile = g_strdup_printf("%s/%s", cfg->statsdir, entry);
-
-   test = G_FILE_TEST_IS_REGULAR | G_FILE_TEST_EXISTS;
-   if(g_file_test (statsfile, test) == FALSE) {
-    g_free(statsfile);
-    statsfile = NULL;
-   }
-   
-   g_free(compare);
-   break;
-  }
-  g_free(compare);
- }
-
- g_dir_close(dir);
-
- return(statsfile);
-}
-
-
-gchar *getStatsFileName(const gchar *hostname)
-{
- gchar *statsfile = NULL;
-
- if(hostname) {
-  statsfile = g_strdup_printf("%s/%s.stat", cfg->statsdir, hostname);
+ if(g_file_test(statsfile, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_EXISTS) == FALSE) {
+   g_free(statsfile);
+   statsfile = NULL;
  }
 
  return(statsfile);
 }
 
 
-gboolean removeStatsFile(const gchar *hostname)
+gchar *getStatsFileName(const HostNode *n)
 {
- gboolean r = FALSE;
- gchar *statsfile;
+ return g_strdup_printf("%s/%s:%d.stat", cfg->statsdir, n->hostname, n->ssh_port);
+}
 
- if(hostname) {
-  statsfile = getStatsFileName(hostname);
 
-  if(!g_unlink(statsfile)) r = TRUE;
+gboolean prepareStatsFile(HostNode *n)
+{
+ gchar *statsfile = getStatsFileName(n);
 
-  g_free(statsfile);  
- }
+ g_unlink(statsfile);
 
- return(r);
+ n->fpstat = fopen(statsfile, "wx");
 
+ g_free(statsfile);
+
+ return n->fpstat != NULL;
 }
 
 
 void refreshStatsOfNode(gpointer n)
 {
+ if( ((HostNode *)n)->fpstat ) {
+    fclose(((HostNode *)n)->fpstat);
+    ((HostNode *)n)->fpstat = NULL;
+ }
+
  getUpdatesFromStat(((HostNode *) n));
 
  unsetLockForHost((HostNode *) n);
@@ -118,26 +89,21 @@ void refreshStatsOfNode(gpointer n)
 gboolean setStatsFileFromIOC(GIOChannel *ioc, GIOCondition condition,
 			     gpointer n)
 {
- gchar *statsfile = NULL;
  gchar *buf = NULL;
  GError *error = NULL;
  gsize bytes;
  gboolean r = TRUE;
- FILE *fp = NULL;
  GIOStatus iostatus =  G_IO_STATUS_NORMAL;
 
  if(!n) return(FALSE);
 
- statsfile = getStatsFileName(((HostNode *) n)->hostname);
-
  if (condition & (G_IO_HUP | G_IO_PRI | G_IO_IN))
   {
    buf = (gchar *) g_malloc0 (g_io_channel_get_buffer_size (ioc) + 1);
 
    r = FALSE;
-   fp = fopen(statsfile, "a");
 
-   while(iostatus == G_IO_STATUS_NORMAL && fp) {
+   while(iostatus == G_IO_STATUS_NORMAL && ((HostNode *) n)->fpstat) {
     iostatus = g_io_channel_read_chars (ioc, buf,
 					g_io_channel_get_buffer_size (ioc), 
 					&bytes, NULL);
@@ -145,13 +111,13 @@ gboolean setStatsFileFromIOC(GIOChannel *ioc, GIOCondition condition,
     if(iostatus == G_IO_STATUS_ERROR || iostatus == G_IO_STATUS_AGAIN)
      break;
 
-    fwrite(buf, sizeof(gchar), bytes, fp);
-    fflush(fp);
+    fwrite(buf, sizeof(gchar), bytes, ((HostNode *) n)->fpstat);
 
     r = TRUE;
    }
 
-   if(r == TRUE) fclose(fp);
+   if(r == TRUE)
+    fflush(((HostNode *) n)->fpstat);
 
    g_free(buf);
   }
@@ -168,8 +134,6 @@ gboolean setStatsFileFromIOC(GIOChannel *ioc, GIOCondition condition,
    r = FALSE;
   }
 
- g_free(statsfile);
-
  return(r);
 }
 
@@ -189,6 +153,9 @@ void freePkgNode(PkgNode *n)
 static void freePackages(HostNode *n)
 {
  if(n && n->packages) {
+#ifdef FEAT_AUTOREF
+  autoref_rem_host_info(n);
+#endif
   g_list_foreach(n->packages, (GFunc) freePkgNode, NULL);
   g_list_free(n->packages);
   n->packages = NULL;
@@ -206,21 +173,27 @@ gboolean getUpdatesFromStat(HostNode *n)
 {
  gchar *statsfile;
  char line[STATS_MAX_LINE_LEN];
- char buf[256];
+ char buf[256] = "\0";
  FILE  *fp;
  PkgNode *pkgnode = NULL;
- gint status, i;
+ gint status=0, i;
  gchar **argv = NULL;
 
  if(!n) return (FALSE);
 
- if(!(statsfile = getStatsFile(n->hostname))) {
-  n->category = C_NO_STATS;
+ if(!(statsfile = getStatsFile(n))) {
+  n->category = C_UNKNOWN;
   return (TRUE);
  }
 
+#ifdef FEAT_AUTOREF
+ struct stat sbuf;
+ if(!stat(statsfile, &sbuf))
+  n->last_upd = sbuf.st_mtime;
+#endif
+
  if(!(fp = (FILE *) g_fopen(statsfile, "r"))) {
-  n->category = C_UNKNOW;
+  n->category = C_UNKNOWN;
   g_free(statsfile);
   return (TRUE);
  }
@@ -229,6 +202,8 @@ gboolean getUpdatesFromStat(HostNode *n)
  n->nupdates = 0;
  n->nextras = 0;
  n->nholds = 0;
+ n->nbrokens = 0;
+ n->forbid = 0;
 
  freePackages(n);
 
@@ -244,6 +219,14 @@ gboolean getUpdatesFromStat(HostNode *n)
   g_free(n->lsb_codename);
   n->lsb_codename = NULL;
  }
+ if(n->uname_kernel) {
+  g_free(n->uname_kernel);
+  n->uname_kernel = NULL;
+ }
+ if(n->uname_machine) {
+  g_free(n->uname_machine);
+  n->uname_machine = NULL;
+ }
  if(n->virt) {
   g_free(n->virt);
   n->virt = NULL;
@@ -255,9 +238,10 @@ gboolean getUpdatesFromStat(HostNode *n)
 
  int linesok = 0;
  while(fgets(line, STATS_MAX_LINE_LEN, fp)) {
-  line[strlen(line) - 1] = 0;
+  /* Remove any whitespace from the line. */
+  g_strchomp(line);
 
-  if (sscanf((gchar *) line, "KERNELINFO: %d %255s", &status, buf)) {
+  if (sscanf((gchar *) line, "KERNELINFO: %d %255s", &status, buf) && !n->kernelrel) {
    n->kernelrel = g_strdup(buf);
    switch(status){
    case 1:
@@ -270,7 +254,7 @@ gboolean getUpdatesFromStat(HostNode *n)
    linesok++;
    continue;
   }
-  
+
   if(!strncmp("STATUS: ", line, 8)) {
     argv = g_strsplit(&line[8], "|", 0);
 
@@ -278,17 +262,20 @@ gboolean getUpdatesFromStat(HostNode *n)
 
     i=0;
     while(argv[i]) i++;
-    
+
     /* ignore invalid lines */
     if(i < 3) {
      g_strfreev(argv);
-     continue;     
+     continue;
     }
-    
+
     pkgnode = g_new0(PkgNode, 1);
+#ifndef NDEBUG
+    pkgnode->_type = T_PKGNODE;
+#endif
     pkgnode->package = g_strdup(argv[0]);
     pkgnode->version = g_strdup(argv[1]);
-   
+
     if (strlen(argv[2]) > 3)
 	pkgnode->data = g_strdup(&argv[2][2]);
 
@@ -308,6 +295,19 @@ gboolean getUpdatesFromStat(HostNode *n)
 	    pkgnode->flag = HOST_STATUS_PKGEXTRA;
 	    n->nextras++;
 	    break;
+	case 'b':
+	    n->status = n->status | HOST_STATUS_PKGBROKEN;
+	    pkgnode->flag = HOST_STATUS_PKGBROKEN;
+	    n->nbrokens++;
+	    break;
+	case 'i':
+	    break;
+	default:
+	 g_free(pkgnode->package);
+	 g_free(pkgnode->version);
+	 g_free(pkgnode);
+	 g_strfreev(argv);
+	 continue;
     }
     g_strfreev(argv);
 
@@ -321,14 +321,14 @@ gboolean getUpdatesFromStat(HostNode *n)
    argv = g_strsplit(&line[8], "|", 0);
 
    if(!argv) continue;
-   
+
    i=0;
    while(argv[i]) i++;
-   
+
    /* ignore invalid lines */
    if(i < 3) {
     g_strfreev(argv);
-    continue;     
+    continue;
    }
 
    if(strlen(argv[0]) > 0) {
@@ -340,7 +340,7 @@ gboolean getUpdatesFromStat(HostNode *n)
    g_strfreev(argv);
 
    linesok++;
-   continue;     
+   continue;
   }
 
   if (sscanf((gchar *) line, "VIRT: %255s", buf)) {
@@ -352,16 +352,37 @@ gboolean getUpdatesFromStat(HostNode *n)
    linesok++;
    continue;
   }
+
+  if (sscanf((gchar *) line, "UNAME: %255s", buf)) {
+   gchar *s = strchr(buf, '|');
+   if (s) {
+    s[0] = 0;
+
+    n->uname_kernel = g_strdup(buf);
+    n->uname_machine = g_strdup(s+1);
+
+    linesok++;
+   }
+   continue;
+  }
+
+  if (sscanf((gchar *) line, "FORBID: %d", &n->forbid))
+   continue;
  }
 
  if(linesok>5) {
-   if(n->status & HOST_STATUS_PKGUPDATE)
+   if (n->status & HOST_STATUS_PKGBROKEN)
+    n->category = C_BROKEN_PKGS;
+   else if(n->status & HOST_STATUS_PKGUPDATE)
     n->category = C_UPDATES_PENDING;
    else
     n->category = C_UP_TO_DATE;
+#ifdef FEAT_AUTOREF
+    autoref_add_host_info(n);
+#endif
  }
  else
-    n->category = C_UNKNOW;
+    n->category = C_UNKNOWN;
 
  fclose(fp);
  g_free(statsfile);
@@ -374,17 +395,28 @@ gboolean refreshStats(GList *hosts)
 {
  GList *ho;
  gboolean r = TRUE;
+ static gboolean upd_pending = FALSE;
+ gboolean n_upd_pending = FALSE;
 
  ho = g_list_first(hosts);
- 
+
+#ifdef FEAT_AUTOREF
+ static gboolean do_autoref = TRUE;
+ gint num_in_refresh = 0;
+#endif
  while(ho) {
   HostNode *n = (HostNode *) ho->data;
 
   if(screen_get_sessions(n)) {
-   n->category = C_SESSIONS;
-   rebuilddl = TRUE;
+   if(n->category != C_SESSIONS) {
+    n->category = C_SESSIONS;
+    rebuilddl = TRUE;
+   }
   }
   else {
+   if(n->category == C_UPDATES_PENDING)
+    n_upd_pending = TRUE;
+
    if (n->category == C_SESSIONS)
     n->category = C_REFRESH_REQUIRED;
 
@@ -401,7 +433,7 @@ gboolean refreshStats(GList *hosts)
     else if (rsetlck == 0) {
      if(n->status & HOST_STATUS_LOCKED) {
       refreshStatsOfNode(n);
-      //      n->status ^= HOST_STATUS_LOCKED;
+      /* n->status ^= HOST_STATUS_LOCKED; */
      } else {
       n->category = C_REFRESH;
       rebuilddl = TRUE;
@@ -409,17 +441,22 @@ gboolean refreshStats(GList *hosts)
       freePackages(n);
 
       if(ssh_cmd_refresh(n) == FALSE) {
-       n->category = C_NO_STATS;
+       n->category = C_UNKNOWN;
       }
      }
     /* Something weird happend. */
     } else {
-     n->category = C_UNKNOW;
+     n->category = C_UNKNOWN;
      rebuilddl = TRUE;
     }
    }
   }
 
+#ifdef FEAT_AUTOREF
+  if(n->category == C_REFRESH)
+   num_in_refresh++;
+#endif
+
   ho = g_list_next(ho);
  }
 
@@ -427,6 +464,24 @@ gboolean refreshStats(GList *hosts)
  applyFilter(hosts);
 #endif
 
+ if(n_upd_pending && !upd_pending)
+  notifyUser();
+ upd_pending = n_upd_pending;
+
+#ifdef FEAT_AUTOREF
+ if(cfg->auto_refresh) {
+  if (num_in_refresh == 0) {
+   if(do_autoref) {
+    if (autoref_trigger_auto())
+     drawStatus (_("Auto refresh triggered..."), FALSE);
+    do_autoref = FALSE;
+   }
+  }
+  else
+   do_autoref = TRUE;
+ }
+#endif
+
  return(r);
 }
 
@@ -445,7 +500,7 @@ void getOldestMtime(GList *hosts)
  while(ho) {
   n = (HostNode *) ho->data;
 
-  if((statsfile = getStatsFile(n->hostname)))
+  if((statsfile = getStatsFile(n)))
    {
     if(!stat(statsfile, &stat_p)) {
      if(difftime(stat_p.st_mtime, oldest_st_mtime) < 0)
